<?php
namespace Neklo\Scheduler\Block\Adminhtml\Schedule;

use \Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\CollectionFactory
        as TimelineCollectionFactory;
use Neklo\Scheduler\Model\Schedule\Filters;

class Timeline extends \Magento\Backend\Block\Template
{
    /**
     *
     * @var TimelineCollectionFactory
     */
    protected $timelineCollectionFactory;
    protected $filters;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        TimelineCollectionFactory $collection,
        Filters $filters,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->timelineCollectionFactory = $collection;
        $this->filters = $filters;
    }
    /**
     *
     * @return \Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\Collection
     */
    public function getJobs()
    {
        $jobs = $this->timelineCollectionFactory->create();
        $this->applyFilters($jobs);
        return $jobs;
    }

    /**
     *
     * @param \Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\Collection $collection
     */
    protected function applyFilters(
        \Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\Collection $collection
    ) {
        $filters = $this->filters;
        $col = $collection;
        $setStart = $setEnd = false;

        $data = $filters->getData();
        foreach ($data as $name => $data) {
            switch(true) {
                case $name != 'show_jobs'://dates
                    if (isset($data['date_from']) && $data['date_from']) {
                        $this->setData('start_time', strtotime($data['date_from']));
                        $col->addFieldToFilter($name.'_at', [
                            'gteq' => date("Y-m-d H:i:s",$this->getData('start_time'))]);
                    }
                    if (isset($data['date_to']) && $data['date_to']) {
                        $this->setData('end_time', strtotime($data['date_to']));
                        $col->addFieldToFilter($name.'_at', [
                            'lteq' => date("Y-m-d H:i:s",$this->getData('end_time'))]);
                    }
                    break;
                default://job names
                    if (!empty($data)) {
                        $col->addFieldToFilter('job_code', ['in' => $data]);
                    }
                    break;
            }
        }
    }

    public function getDataSource($collection)
    {
        $ret = [];
        foreach ($collection->getItems() as $item) {
            $ret[$item->getId().''] = $item->toArray();
        }
        return json_encode($ret);
    }

    public function getStarttime($collection)
    {
        if (!$this->getData('start_time')) {
            $coll = $collection;
            $start = PHP_INT_MAX;
            foreach ($coll as $rec) {
                if ($rec->getScheduledAt()){
                    $tmp = strtotime($rec->getScheduledAt());
                    if ($tmp < $start) $start = $tmp;
                }
                if ($rec->getExecutedAt()){
                    $tmp = strtotime($rec->getExecutedAt());
                    if ($tmp < $start) $start = $tmp;
                }
                if ($rec->getFinishedAt()){
                    $tmp = strtotime($rec->getFinishedAt());
                    if ($tmp < $start) $start = $tmp;
                }
            }
            $this->setData('start_time', $start);
        }
        return $this->getData('start_time');
    }

    public function getEndtime($collection)
    {
        if (!$this->getData('end_time')) {
            $coll = $collection;
            $start = 0;
            foreach ($coll as $rec) {
                if ($rec->getScheduledAt()){
                    $tmp = strtotime($rec->getScheduledAt());
                    if ($tmp > $start) $start = $tmp;
                }
                if ($rec->getExecutedAt()){
                    $tmp = strtotime($rec->getExecutedAt());
                    if ($tmp > $start) $start = $tmp;
                }
                if ($rec->getFinishedAt()){
                    $tmp = strtotime($rec->getFinishedAt());
                    if ($tmp > $start) $start = $tmp;
                }
            }
            $this->setData('end_time', $start);
        }
        return $this->getData('end_time');
    }

    public function decorateTime($time, $format)
    {
        $time *= 1;
//        $day = substr(date('l', $time), 3);
//        return $day . '&nbsp;' . date('j.m.', $time);
        return date($format, $time);
    }
}
